@@ -36,7 +36,7 @@ func TestEscapeAttrVal(t *testing.T) {
 			if len(b) > 1 && (b[0] == '"' || b[0] == '\'') && b[0] == b[len(b)-1] {
 				b = b[1 : len(b)-1]
 			}
-			val := EscapeAttrVal(&buf, b, quote, false, false)
+			val := EscapeAttrVal(&buf, b, quote, false)
 			test.String(t, string(val), tt.expected)
 		})
 	}
@@ -48,9 +48,9 @@ func TestEscapeAttrValXML(t *testing.T) {
 		expected string
 	}{
 		{`"xyz"`, `"xyz"`},
-		{`xyz`, `"xyz"`},
-		{`'xyz'`, `"xyz"`},
-		{``, `""`},
+		{`'xyz'`, `'xyz'`},
+		{`xyz`, `xyz`},
+		{``, ``},
 	}
 	var buf []byte
 	for _, tt := range escapeAttrValTests {
@@ -63,7 +63,7 @@ func TestEscapeAttrValXML(t *testing.T) {
 			if len(b) > 1 && (b[0] == '"' || b[0] == '\'') && b[0] == b[len(b)-1] {
 				b = b[1 : len(b)-1]
 			}
-			val := EscapeAttrVal(&buf, b, quote, false, true)
+			val := EscapeAttrVal(&buf, b, quote, true)
 			test.String(t, string(val), tt.expected)
 		})
 	}
