@@ -80,6 +80,7 @@ func (l *Lexer) RegExp() (TokenType, []byte) {
 
 // Next returns the next Token. It returns ErrorToken when an error was encountered. Using Err() one can retrieve the error message.
 func (l *Lexer) Next() (TokenType, []byte) {
+	l.err = nil // clear error from previous ErrorToken
 	prevLineTerminator := l.prevLineTerminator
 	l.prevLineTerminator = false
 
@@ -220,6 +221,7 @@ func (l *Lexer) Next() (TokenType, []byte) {
 
 	r, _ := l.r.PeekRune(0)
 	l.err = parse.NewErrorLexer(l.r, "unexpected %s", parse.Printable(r))
+	l.r.MoveRune() // allow to continue after error
 	return ErrorToken, l.r.Shift()
 }
 
