@@ -259,6 +259,9 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 				p.fail("let declaration")
 				return
 			}
+		} else if p.tt == OpenBracketToken {
+			p.failMessage("unexpected let [ in single-statement context")
+			return
 		} else {
 			// expression
 			stmt = &ExprStmt{p.parseIdentifierExpression(OpExpr, let)}
@@ -520,13 +523,13 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 		}
 		stmt = p.parseFuncDecl()
 	case AsyncToken: // async function
-		if !allowDeclaration {
-			p.fail("statement")
-			return
-		}
 		async := p.data
 		p.next()
 		if p.tt == FunctionToken && !p.prevLT {
+			if !allowDeclaration {
+				p.fail("statement")
+				return
+			}
 			stmt = p.parseAsyncFuncDecl()
 		} else {
 			// expression
