@@ -165,6 +165,8 @@ func TestParse(t *testing.T) {
 		{"async();", "Stmt(async())"},
 		{"async(a=6, ...b)", "Stmt(async((a=6), ...b))"},
 		{"async(function(){})", "Stmt(async(Decl(function Params() Stmt({ }))))"},
+		{"async.value", "Stmt(async.value)"},
+		{"if(a)async.value", "Stmt(if a Stmt(async.value))"},
 		{"function a(){ async\nawait => b }", "Decl(function a Params() Stmt({ Stmt(async) Stmt(Params(Binding(await)) => Stmt({ Stmt(return b) })) }))"},
 		{"a + async\nb", "Stmt(a+async) Stmt(b)"},
 		{"a + async\nfunction f(){}", "Stmt(a+async) Decl(function f Params() Stmt({ }))"},
@@ -579,7 +581,7 @@ func TestParseError(t *testing.T) {
 
 		// no declarations
 		{"if(a) function f(){}", "unexpected function in statement"},
-		{"if(a) async function f(){}", "unexpected async in statement"},
+		{"if(a) async function f(){}", "unexpected function in statement"},
 		{"if(a) class c{}", "unexpected class in statement"},
 
 		// yield, async, await
@@ -724,7 +726,8 @@ func TestParseError(t *testing.T) {
 		{"\u2010", "unexpected \u2010"},
 		{"a=\u2010", "unexpected \u2010 in expression"},
 		{"/", "unexpected EOF or newline in regular expression"},
-		{"({...[]})=>a", "unexpected => in expression"}, // go-fuzz
+		{"({...[]})=>a", "unexpected => in expression"},                   // go-fuzz
+		{`for(;;)let[a]`, `unexpected let [ in single-statement context`}, // go-fuzz
 	}
 	for _, tt := range tests {
 		t.Run(tt.js, func(t *testing.T) {
